;ex1.asm
;Purpose:	Example for usage with shell32.
;Build:		It assembles now with TASM, MASM, and JWASM.
;               tasm /ml /m2 /q ex1.asm
;		jwasm -c ex1.asm
;		ml /c ex1.asm 
;               Link it with a DOS linker, which can link 16 and 32 bit segments into a MZ EXE.
;               For example: tlink /3 /x shell32.obj ex1.obj,ex1.exe
;Author:	Gunther
;Note:		Give credit where credit is due.

                .386p

		public	Main

code32          segment para public use32
                assume  cs:code32, ds:code32, ss:code32

;----- Data

hello		db	"***** Hello from the 32-bit FLAT segment! *****"
hello_len	equ	$ - hello

cont		db	"Please press any key to end the application ..."
cont_len	equ	$ - cont

;----- Code

Main		proc	near
		call	Cls			;clear screen
                push	25
                push	0
                call    SetCursor		;hide cursor
                push	offset hello
                push	12
                push	16
                push	hello_len
                call    PokeString		;say Hello
                push	offset cont
                push	23
                push	0
                push	cont_len
                call	PokeString		;print end message
		call	Waiting			;wait
                push	0
                push	0
                call    SetCursor		;make cursor visible
                call	Cls			;clear screen
		ret
Main		endp

;Waiting
;Purpose:	Wait for key.
;Input:		None
Waiting         proc    near
		push	eax
		xor	eax,eax			;ah = 0 ===> FUNCTION: keyboard read
		int	16h			;transfer to BIOS keyboard interrupt
		pop	eax
                ret
Waiting         endp

;Cls
;Purpose:	Clear the VGA compatible text screen.
;Input:		None
Cls		proc	near
		push	eax
		push	ebx
		push	ecx
		push	edx
		xor	eax,eax
		xor	ecx,ecx			;ch = left upper row
						;cl = left upper column
		mov	dh,24			;dh = right bottom row
		mov	dl,79			;dl = right bottom column
		mov	bh,07h			;bh = screen attribute
		mov	ah,06h			;FUNCTION: scroll up
						;al = number of rows to scroll
		int	10h			;transfer to BIOS video interrupt
		pop	edx
		pop	ecx
		pop	ebx
		pop	eax
		ret
Cls		endp

;SetCursor
;Task:          Set cursor position in text mode.
;Input:         [ebp+8]  = row (0 is top line)
;               [ebp+12] = column (0 is far left)
;Output:        new cursor position
SetCursor       proc    near
                push    ebp
                mov     ebp,esp
                push    eax
                push    ecx
                push    edx
                mov	ah,0fh			;FUNCTION: get display page number
                int	10h			;transfer to BIOS video interrupt (result in bh)
                xor	ax,ax
                mov	ah,02h			;FUNCTION: set cursor position
                mov     edx,[ebp+8]		;dl = column
                mov     ecx,[ebp+12]		;cl = row
                mov     dh,cl           	;dh = row
                int	10h			;transfer to BIOS video interrupt
                pop     edx
                pop     ecx
                pop     eax
                pop     ebp
                ret     8
SetCursor       endp

;PokeString
;Task:          Poke a string into the VGA text buffer.
;Input:         [ebp+8]  = string length
;               [ebp+12] = column
;               [ebp+16] = row
;               [ebp+20] = string offset
;Output:        string on the screen
PokeString      proc    near
                push    ebp
                mov     ebp,esp
                push    eax
                push    ebx
                push    ecx
                push    edi
                mov     edi,0b8000h		;gs:edi -> start text buffer
                mov     ebx,[ebp+20]		;ds:ebx -> string to print
                mov     ecx,[ebp+8]		;ecx = string length
                mov     eax,80			;eax = screen width
                imul    eax,[ebp+16]		;eax = row*width
                add     eax,[ebp+12]		;eax = row*width+column
                shl     eax,1           	;eax = (row*width+column)*2
                add     edi,eax         	;gs:edi -> destination
                mov     ah,07h			;ah = screen attribute

PokeString1:

		mov     al,ds:[ebx]     	;al = character to poke
                mov     word ptr gs:[edi],ax	;poke character & attribute
                inc     ebx             	;update pointers
                add     edi,2
                dec     ecx             	;decrement loop counter
                jne     PokeString1
                pop     edi
                pop     ecx
                pop     ebx
                pop     eax
                pop     ebp
                ret     16
PokeString      endp

code32          ends
                end